@@ -11,6 +11,7 @@ package main
 import (
 	"encoding/binary"
 	"errors"
+	"runtime"
 	"sync"
 	"sync/atomic"
 	"time"
@@ -92,14 +93,14 @@ var (
 )
 
 // Initialize low-level USB I/O
-func UsbInit() error {
-	_, err := libusbContext()
+func UsbInit(nopnp bool) error {
+	_, err := libusbContext(nopnp)
 	return err
 }
 
 // libusbContext returns libusb_context. It
 // initializes context on demand.
-func libusbContext() (*C.libusb_context, error) {
+func libusbContext(nopnp bool) (*C.libusb_context, error) {
 	if atomic.LoadInt32(&libusbContextOk) != 0 {
 		return libusbContextPtr, nil
 	}
@@ -114,21 +115,24 @@ func libusbContext() (*C.libusb_context, error) {
 	}
 
 	// Subscribe to hotplug events
-	C.libusb_hotplug_register_callback(
-		libusbContextPtr, // libusb_context
-		C.LIBUSB_HOTPLUG_EVENT_DEVICE_ARRIVED| // events mask
-			C.LIBUSB_HOTPLUG_EVENT_DEVICE_LEFT,
-		C.LIBUSB_HOTPLUG_NO_FLAGS,  // flags
-		C.LIBUSB_HOTPLUG_MATCH_ANY, // vendor_id
-		C.LIBUSB_HOTPLUG_MATCH_ANY, // product_id
-		C.LIBUSB_HOTPLUG_MATCH_ANY, // dev_class
-		C.libusb_hotplug_callback_fn(unsafe.Pointer(C.libusbHotplugCallback)),
-		nil, // callback's data
-		nil, // deregister handle
-	)
+	if !nopnp {
+		C.libusb_hotplug_register_callback(
+			libusbContextPtr, // libusb_context
+			C.LIBUSB_HOTPLUG_EVENT_DEVICE_ARRIVED| // events mask
+				C.LIBUSB_HOTPLUG_EVENT_DEVICE_LEFT,
+			C.LIBUSB_HOTPLUG_NO_FLAGS,  // flags
+			C.LIBUSB_HOTPLUG_MATCH_ANY, // vendor_id
+			C.LIBUSB_HOTPLUG_MATCH_ANY, // product_id
+			C.LIBUSB_HOTPLUG_MATCH_ANY, // dev_class
+			C.libusb_hotplug_callback_fn(unsafe.Pointer(C.libusbHotplugCallback)),
+			nil, // callback's data
+			nil, // deregister handle
+		)
+	}
 
 	// Start libusb thread (required for hotplug)
 	go func() {
+		runtime.LockOSThread()
 		for {
 			C.libusb_handle_events(libusbContextPtr)
 		}
@@ -174,7 +178,7 @@ func UsbCheckIppOverUsbDevices() bool {
 // device descriptors
 func UsbGetIppOverUsbDeviceDescs() (map[UsbAddr]UsbDeviceDesc, error) {
 	// Obtain libusb context
-	ctx, err := libusbContext()
+	ctx, err := libusbContext(false)
 	if err != nil {
 		return nil, err
 	}
@@ -305,7 +309,7 @@ type UsbDevHandle C.libusb_device_handle
 // UsbOpenDevice opens device by device descriptor
 func UsbOpenDevice(desc UsbDeviceDesc) (*UsbDevHandle, error) {
 	// Obtain libusb context
-	ctx, err := libusbContext()
+	ctx, err := libusbContext(false)
 	if err != nil {
 		return nil, err
 	}
@@ -612,7 +616,7 @@ func (iface *UsbInterface) Close() {
 //    does never happen that the USB addressing and configuration changes. What
 //    is actually done is that all buffers get flushed and the bulk IN and OUT
 //    pipes get reset to their default states. This clears all stall conditions.
-//    See http://cholla.mmto.org/computers/linux/usb/usbprint11.pdf
+//    See http://cholla.mmto.org/computers/linux/usb/usbprint11.
 func (iface *UsbInterface) SoftReset() error {
 	rc := C.libusb_control_transfer(
 		(*C.libusb_device_handle)(iface.devhandle),
